package org.ogce.xregistry.client;

import java.security.cert.X509Certificate;

import org.ietf.jgss.GSSCredential;
import org.ogce.xregistry.security.ApplicationProperties;
import org.ogce.xregistry.utils.XRegistryClientException;

public class XRegistryClientUtil {
	/**
	 * If Xregistry service is running non-secure mode means if its over http.
	 * @param registryServiceUrl
	 * @return Object of DocumentRegistryClient which have all methods
	 * @throws XRegistryClientException
	 */
	public static XRegistryClient CreateNonSecureRegistryInstance(String registryServiceUrl) throws XRegistryClientException {
			XRegistryClient registryClient = new XRegistryClient(registryServiceUrl);
			return registryClient;
	}
		
	/**
	 * Provide the location and name of the file which have all your properties.
	 * You can find xregistry.properties as reference property file in config folder.
	 * @param propertyfile
	 * @return Object of DocumentRegistryClient which have all methods
	 * @throws XRegistryClientException
	 */
	public static XRegistryClient CreateGSISecureRegistryInstance(String propertyfile) throws XRegistryClientException {
		ApplicationProperties properties = new ApplicationProperties(propertyfile);
		XRegistryClient registryClient = new XRegistryClient(properties.getGssCredential(), properties.getCredentials().getTrustedCertificates(), properties.getXRegistryUrl());
		return registryClient;
	}

	/**
	 * If you have hostcerificate and trusted certificates to create a secure connection.
	 * @param hostcertsKeyFile
	 * @param trustedcertsFile
	 * @param registryServiceUrl
	 * @return Object of DocumentRegistryClient which have all methods
	 * @throws XRegistryClientException
	 */
	public static XRegistryClient CreateSSLSecureRegistryInstance(String hostcertsKeyFile, String trustedcertsFile, String registryServiceUrl) throws XRegistryClientException {
		ApplicationProperties properties = new ApplicationProperties(hostcertsKeyFile, trustedcertsFile);
		XRegistryClient registryClient = new XRegistryClient(properties.getCredentials()
				.getHostcertsKeyFile(), properties.getCredentials().getTrustedCertificates(), registryServiceUrl);
		return registryClient;
	}
	/**
	 * Provide all the myproxy parameter to create GSSCredential and provide xregistry URL to connect to 
	 * @param myproxyServer
	 * @param myproxyPort
	 * @param myproxyUsername
	 * @param myproxyPassphrase
	 * @param myproxyLifetime
	 * @param trustedcertsFile
	 * @param registryServiceUrl
	 * @return Object of DocumentRegistryClient which have all methods
	 * @throws XRegistryClientException
	 */
	public static XRegistryClient CreateMyProxyCredentialRegistryInstance(String myproxyServer, int myproxyPort, String myproxyUsername, String myproxyPassphrase, int myproxyLifetime, String trustedcertsFile, String registryServiceUrl)
			throws XRegistryClientException {
		ApplicationProperties properties = new ApplicationProperties(myproxyServer, myproxyPort, myproxyUsername, myproxyPassphrase, myproxyLifetime, trustedcertsFile);
		XRegistryClient registryClient = new XRegistryClient(properties.getGssCredential(), properties.getCredentials().getTrustedCertificates(),
				registryServiceUrl);
		return registryClient;
	}
	/**
	 * provide GSSCredential credentials and certificates location along with xregistry path 
	 * @param credential
	 * @param trustedcertsFile
	 * @param registryServiceUrl
	 * @return Object of DocumentRegistryClient which have all methods
	 * @throws XRegistryClientException
	 */
	public static XRegistryClient CreateGSISecureRegistryInstance(GSSCredential credential, String trustedcertsFile, String registryServiceUrl) throws XRegistryClientException {
		 X509Certificate[] certificates = ApplicationProperties.loadCeritificates(trustedcertsFile) ;
		XRegistryClient registryClient = new XRegistryClient(credential, certificates, registryServiceUrl);
		return registryClient;
	}
	
}
