/*
 * Copyright (c) 2009 Pervasive Lab, Indiana University. All rights reserved.
 *
 * This software is open source. See the bottom of this file for the license.
 *
 * $Id: $
 */
package org.ogce.xregistry.security;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.Serializable;
import java.security.cert.X509Certificate;

import org.globus.gsi.GlobusCredential;
import org.globus.gsi.GlobusCredentialException;
import org.globus.gsi.TrustedCertificates;
import org.globus.gsi.gssapi.GlobusGSSCredentialImpl;
import org.globus.myproxy.MyProxy;
import org.ietf.jgss.GSSCredential;
import org.ietf.jgss.GSSException;
import org.ogce.xregistry.utils.XRegistryClientException;

/**
 * @author Raminderjeet Singh
 */
public class MyProxyCredentials implements Serializable {


	private static final long serialVersionUID = -2471014486509046212L;
	protected String myproxyHostname;
	protected String myproxyUserName;
	protected String myproxyPassword;
	protected int myproxyPortNumber;
	protected GSSCredential gssCredential;
	protected String portalUserName;
	private String hostcertsKeyFile;
	private String trustedCertsFile;

	protected int myproxyLifeTime = 14400;
	final static int SECS_PER_MIN = 60;
	final static int SECS_PER_HOUR = 3600;
	private X509Certificate[] trustedCertificates;
	private boolean initialized = false;
	private boolean user = true;

	public MyProxyCredentials() {
		// default constructor
	}

	public MyProxyCredentials(String myproxyServer, int myproxyPort, String myproxyUsername, String myproxyPassphrase, int myproxyLifetime, String trustedCertsFile) {
		this.myproxyHostname = myproxyServer;
		if (myproxyPort == 0) {
			this.myproxyPortNumber = MyProxy.DEFAULT_PORT;
		} else {
			this.myproxyPortNumber = myproxyPort;
		}
		this.myproxyUserName = myproxyUsername;
		this.myproxyPassword = myproxyPassphrase;
		this.myproxyLifeTime = myproxyLifetime;
		this.trustedCertsFile = trustedCertsFile;

	}

	public GSSCredential getGssCredential() throws XRegistryClientException {
		try {
		if (gssCredential == null || gssCredential.getRemainingLifetime() < 15 * 60) {
				if (hostcertsKeyFile != null && !user) {
					init();
					FileInputStream fis = new FileInputStream(hostcertsKeyFile);
					GlobusCredential globusCred = new GlobusCredential(fis);
					this.gssCredential = new GlobusGSSCredentialImpl(globusCred, GSSCredential.INITIATE_AND_ACCEPT);

				} else {
					this.gssCredential = getDefaultProxy();
				}
				if (gssCredential != null) {
					return gssCredential;
				}
			// then we will try the MyProxy
			if (getMyproxyUserName() != null && getMyproxyPassword() != null) {
				gssCredential = renewProxy();
			}
		}
		} catch (Exception e) {
			System.out.println("Failed to load proxy credential from ProxyManager");
			throw new XRegistryClientException(e.getLocalizedMessage(),e);
		}

		return gssCredential;
	}

	public GSSCredential getGssCredential(String hostcertsKeyFile, String trustedcertsKeyFile) throws XRegistryClientException {
		try {
			if (gssCredential == null || gssCredential.getRemainingLifetime() < 15 * 60) {
				loadCertificate(trustedcertsKeyFile);
				FileInputStream fis = new FileInputStream(hostcertsKeyFile);
				GlobusCredential globusCred = new GlobusCredential(fis);
				this.gssCredential = new GlobusGSSCredentialImpl(globusCred, GSSCredential.INITIATE_AND_ACCEPT);
			}
		} catch (FileNotFoundException e) {
			throw new XRegistryClientException(e.getLocalizedMessage(),e);
		} catch (GlobusCredentialException e) {
			throw new XRegistryClientException(e.getLocalizedMessage(),e);
		} catch (GSSException e) {
			throw new XRegistryClientException(e.getLocalizedMessage(),e);
		}
		return gssCredential;
	}

	public GSSCredential getDefaultProxy() throws Exception {
		init();
		MyProxy myproxy = new MyProxy(this.myproxyHostname, this.myproxyPortNumber);
		return myproxy.get(this.myproxyUserName, this.myproxyPassword, this.myproxyLifeTime);
	}

	private void init() {
		loadCertificate(trustedCertsFile);
	}

	public void loadCertificate(String trustedCertsFile) {
		if (trustedCertsFile != null) {
			if (new File(trustedCertsFile).isDirectory()) {
				TrustedCertificates certificates = TrustedCertificates.load(trustedCertsFile);
				TrustedCertificates.setDefaultTrustedCertificates(certificates);
				this.setTrustedCertificates(certificates.getCertificates());
			} else {
				this.setTrustedCertificates(CertificateManager.getTrustedCertificates(trustedCertsFile));

				TrustedCertificates certificatesArray = new TrustedCertificates(this.getTrustedCertificates());
				TrustedCertificates.setDefaultTrustedCertificates(certificatesArray);
			}
		}
	}

	public GSSCredential renewProxy() throws XRegistryClientException {
		init();
		try {
			String proxyloc = null;
			MyProxy myproxy = new MyProxy(myproxyHostname, myproxyPortNumber);
			int lifeHours = myproxyLifeTime * SECS_PER_HOUR;
			GSSCredential proxy = myproxy.get(myproxyUserName, myproxyPassword, lifeHours);

			GlobusCredential globusCred = null;
			if (proxy instanceof GlobusGSSCredentialImpl) {
				globusCred = ((GlobusGSSCredentialImpl) proxy).getGlobusCredential();
				String uid = myproxyUserName;
				if (proxyloc == null) {
					System.out.println("uid: " + uid);
					proxyloc = "/tmp/x509up_u" + uid;
				}
				File proxyfile = new File(proxyloc);
				System.out.println("proxy location: " + proxyfile.getAbsolutePath());
				if (proxyfile.exists() == false) {
					String dirpath = proxyloc.substring(0, proxyloc.lastIndexOf('/'));
					File dir = new File(dirpath);
					if (dir.exists() == false) {
						dir.mkdirs();
						System.out.println("new directory " + dirpath + " is created.");
					}
					proxyfile.createNewFile();
					System.out.println("new proxy file " + proxyloc + " is created.");
				}
				FileOutputStream fout = new FileOutputStream(proxyfile);
				globusCred.save(fout);
				fout.close();
				String osName = System.getProperty("os.name");
				if (!osName.contains("Windows")) {
					Runtime.getRuntime().exec("/bin/chmod 600 " + proxyloc);
				}
				System.out.println("Proxy file renewed to " + proxyloc + " for the user " + myproxyUserName + " with " + myproxyLifeTime + " lifetime.");

			}
			return proxy;
		} catch (Exception e) {
			throw new XRegistryClientException(e.getLocalizedMessage(),e);
		}
	}

	/**
	 * @return the myproxyHostname
	 */
	public String getMyproxyHostname() {
		return myproxyHostname;
	}

	/**
	 * @param myproxyHostname
	 *            the myproxyHostname to set
	 */
	public void setMyproxyHostname(String myproxyHostname) {
		this.myproxyHostname = myproxyHostname;
	}

	/**
	 * @return the myproxyUserName
	 */
	public String getMyproxyUserName() {
		return myproxyUserName;
	}

	/**
	 * @param myproxyUserName
	 *            the myproxyUserName to set
	 */
	public void setMyproxyUserName(String myproxyUserName) {
		this.myproxyUserName = myproxyUserName;
	}

	/**
	 * @return the myproxyPassword
	 */
	public String getMyproxyPassword() {
		return myproxyPassword;
	}

	/**
	 * @param myproxyPassword
	 *            the myproxyPassword to set
	 */
	public void setMyproxyPassword(String myproxyPassword) {
		this.myproxyPassword = myproxyPassword;
	}

	/**
	 * @return the myproxyLifeTime
	 */
	public int getMyproxyLifeTime() {
		return myproxyLifeTime;
	}

	/**
	 * @param myproxyLifeTime
	 *            the myproxyLifeTime to set
	 */
	public void setMyproxyLifeTime(int myproxyLifeTime) {
		this.myproxyLifeTime = myproxyLifeTime;
	}

	/**
	 * @return the myproxyPortNumber
	 */
	public int getMyproxyPortNumber() {
		return myproxyPortNumber;
	}

	/**
	 * @param myproxyPortNumber
	 *            the myproxyPortNumber to set
	 */
	public void setMyproxyPortNumber(int myproxyPortNumber) {
		this.myproxyPortNumber = myproxyPortNumber;
	}

	/**
	 * @return the portalUserName
	 */
	public String getPortalUserName() {
		return portalUserName;
	}

	/**
	 * @param portalUserName
	 *            the portalUserName to set
	 */
	public void setPortalUserName(String portalUserName) {
		this.portalUserName = portalUserName;
	}

	/**
	 * Returns the initialized.
	 * 
	 * @return The initialized
	 */
	public boolean isInitialized() {
		return this.initialized;
	}

	/**
	 * Sets initialized.
	 * 
	 * @param initialized
	 *            The initialized to set.
	 */
	public void setInitialized(boolean initialized) {
		this.initialized = initialized;
	}

	/**
	 * @param hostcertsKeyFile
	 *            the hostcertsKeyFile to set
	 */
	public void setHostcertsKeyFile(String hostcertsKeyFile) {
		this.hostcertsKeyFile = hostcertsKeyFile;
	}

	/**
	 * @return the hostcertsKeyFile
	 */
	public String getHostcertsKeyFile() {
		return hostcertsKeyFile;
	}

	/**
	 * @param trustedCertsFile
	 *            the trustedCertsFile to set
	 */
	public void setTrustedCertsFile(String trustedCertsFile) {
		this.trustedCertsFile = trustedCertsFile;
	}

	/**
	 * @return the trustedCertsFile
	 */
	public String getTrustedCertsFile() {
		return trustedCertsFile;
	}

	public void setTrustedCertificates(X509Certificate[] trustedCertificates) {
		this.trustedCertificates = trustedCertificates;
	}

	public X509Certificate[] getTrustedCertificates() {
		return trustedCertificates;
	}
}
